/******************************************************************************
 JXFileDocument.h

	Interface for the JXFileDocument class

	Copyright  1996-97 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXFileDocument
#define _H_JXFileDocument

#include <JXDocument.h>
#include <jTime.h>

class JString;
class JChooseSaveFile;

class JXFileDocument : public JXDocument
{
public:

	enum FileStatus
	{
		kFileReadable,
		kNeedNewerVersion,
		kNotMyFile
	};

public:

	JXFileDocument(JXDirector* supervisor, const JCharacter* fileName,
				   const JBoolean onDisk, const JBoolean wantBackupFile,
				   const JCharacter* defaultFileNameSuffix);

	virtual ~JXFileDocument();

	virtual JBoolean	NeedsSave() const;
	void				DataModified();
	void				DataReverted();

	JBoolean		ExistsOnDisk() const;
	const JString&	GetFilePath() const;
	const JString&	GetFileName() const;
	JString			GetFullName(JBoolean* onDisk) const;
	virtual JString	GetName() const;

	JBoolean	WillMakeBackupFile() const;
	void		ShouldMakeBackupFile(const JBoolean doIt);

	JBoolean	WillMakeNewBackupEveryOpen() const;
	void		ShouldMakeNewBackupEveryOpen(const JBoolean doIt);

	JBoolean	WillAllocateTitleSpace() const;
	void		ShouldAllocateTitleSpace(const JBoolean doIt);

	virtual void	SafetySave(const JXDocumentManager::SafetySaveReason reason);
	static JBoolean	CheckForSafetySaveFiles(const JCharacter* fullName,
											JPtrArray<JString>* filesToOpen);

	// saving setup information

	void	ReadJXFDSetup(istream& input);
	void	WriteJXFDSetup(ostream& output) const;

	// provides safe downcasting

	virtual JXFileDocument*			CastToJXFileDocument();
	virtual const JXFileDocument*	CastToJXFileDocument() const;

protected:

	void	SaveCopyInNewFile();
	void	SaveInNewFile();
	void	SaveInCurrentFile();
	void	AdjustWindowTitle();
	void	FileChanged(const JCharacter* fileName, const JBoolean onDisk);

	virtual JBoolean	OKToClose();
	virtual JBoolean	OKToRevert();
	virtual JBoolean	CanRevert();
	virtual void		WriteFile(ostream& output, const JBoolean safetySave) const = 0;

	static FileStatus	DefaultCanReadASCIIFile(istream& input,
												const JCharacter* fileSignature,
												const JFileVersion currFileVersion,
												JFileVersion* actualFileVersion);

	void	SetChooseSaveFile(JChooseSaveFile* csf);
	void	SetSaveBeforeClosePrompt(const JCharacter* prompt);
	void	SetSaveNewFilePrompt(const JCharacter* prompt);
	void	SetOKToRevertPrompt(const JCharacter* prompt);

private:

	JString*	itsFilePath;
	JString*	itsFileName;
	JString*	itsFileSuffix;
	JBoolean	itsWasOnDiskFlag;
	JBoolean	itsSavedFlag;
	JBoolean	itsWantBackupFileFlag;			// kTrue if derived class wants backup file
	JBoolean	itsMakeBackupFileFlag;			// kTrue if should make backup before saving
	JBoolean	itsWantNewBackupEveryOpenFlag;	// kTrue if replace backup before first save
	JBoolean	itsIsFirstSaveFlag;				// kTrue if user hasn't saved yet
	time_t		itsFileModTime;
	JBoolean	itsCheckModTimeFlag;
	JBoolean	itsAllocateTitleSpaceFlag;

	JBoolean	itsNeedSafetySaveFlag;	// kTrue if not safety saved after latest DataModified()
	JString*	itsSafetySaveFileName;	// not NULL if safety save file exists

	JChooseSaveFile*	itsCSF;			// we don't own this
	JString*			itsSaveBeforeClosePrompt;
	JString*			itsSaveNewFilePrompt;
	JString*			itsOKToRevertPrompt;

private:

	JString	GetFileNameForSave() const;
	void	RemoveSafetySaveFile();

	// not allowed

	JXFileDocument(const JXFileDocument& source);
	const JXFileDocument& operator=(const JXFileDocument& source);

public:

	// JBroadcaster messages

	static const JCharacter* kNameChanged;

	class NameChanged : public JBroadcaster::Message
		{
		public:

			NameChanged(const JString& fullName)
				:
				JBroadcaster::Message(kNameChanged),
				itsFullName(fullName)
				{ };

			const JString&
			GetFullName() const
			{
				return itsFullName;
			}

		private:

			const JString& itsFullName;
		};
};


/*****************************************************************************
 File path and name

 ******************************************************************************/

inline const JString&
JXFileDocument::GetFilePath()
	const
{
	return *itsFilePath;
}

inline const JString&
JXFileDocument::GetFileName()
	const
{
	return *itsFileName;
}

/******************************************************************************
 DataModified

 ******************************************************************************/

inline void
JXFileDocument::DataModified()
{
	if (itsSavedFlag)
		{
		itsSavedFlag = kFalse;
		AdjustWindowTitle();
		}

	itsNeedSafetySaveFlag = kTrue;
}

/******************************************************************************
 SetChooseSaveFile (protected)

 ******************************************************************************/

inline void
JXFileDocument::SetChooseSaveFile
	(
	JChooseSaveFile* csf
	)
{
	itsCSF = csf;
}

/******************************************************************************
 Make backup file

 ******************************************************************************/

inline JBoolean
JXFileDocument::WillMakeBackupFile()
	const
{
	return itsWantBackupFileFlag;
}

inline void
JXFileDocument::ShouldMakeBackupFile
	(
	const JBoolean doIt
	)
{
	itsWantBackupFileFlag = doIt;
	itsMakeBackupFileFlag = JConvertToBoolean( itsWasOnDiskFlag && itsWantBackupFileFlag );
}

/******************************************************************************
 When to make backup file

	This controls whether a backup file is created only if none exists
	or whenever the file is first saved after being opened.

 ******************************************************************************/

inline JBoolean
JXFileDocument::WillMakeNewBackupEveryOpen()
	const
{
	return itsWantNewBackupEveryOpenFlag;
}

inline void
JXFileDocument::ShouldMakeNewBackupEveryOpen
	(
	const JBoolean doIt
	)
{
	itsWantNewBackupEveryOpenFlag = doIt;
}

/******************************************************************************
 Allocate space for "***" in window title

 ******************************************************************************/

inline JBoolean
JXFileDocument::WillAllocateTitleSpace()
	const
{
	return itsAllocateTitleSpaceFlag;
}

inline void
JXFileDocument::ShouldAllocateTitleSpace
	(
	const JBoolean doIt
	)
{
	itsAllocateTitleSpaceFlag = doIt;
	AdjustWindowTitle();
}

#endif
