/******************************************************************************
 ItemCost.cc

								The ItemCost Class

	This class stores a list of Item Code vs Cost in a hash table.
	The Item Code must be 4 characters long.

	BASE CLASS = JFileHashTable<JString>

	Written by John Lindal.

 ******************************************************************************/

#include "ItemCost.h"
#include <JString.h>
#include <JKLRand.h>
#include <iostream.h>
#include <jAssert.h>

enum
{
	kSlotCount  = 4,

	kKeyLength  = 4,
	kCostLength = sizeof(JFloat)
};

/******************************************************************************
 Constructor

 ******************************************************************************/

ItemCost::ItemCost()
	:
	JFileHashTable<JString>()
{
	itsLockForIterFlag = kFalse;
	itsHashTableIndex  = 0;

	OpenFile("FHT.test.data", sizeof(JSize), kSlotCount, kKeyLength, kCostLength);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

ItemCost::~ItemCost()
{
}

/******************************************************************************
 AddItem

 ******************************************************************************/

void
ItemCost::AddItem
	(
	const JString&	theKey,
	const JFloat	cost
	)
{
	assert( !itsLockForIterFlag );
	assert( theKey.GetLength() == kKeyLength );

	FindElement(theKey, kTrue);
	WriteCost(cost);
}

/******************************************************************************
 RemoveItem

 ******************************************************************************/

void
ItemCost::RemoveItem
	(
	const JString& theKey
	)
{
	assert( !itsLockForIterFlag );

	JFileHashTable<JString>::RemoveElement(theKey);
}

/******************************************************************************
 GetItemCost

	Returns kTrue if the item is in the list.

 ******************************************************************************/

JBoolean
ItemCost::GetItemCost
	(
	const JString&	theKey,
	JFloat*			cost
	)
{
	assert( !itsLockForIterFlag );
	assert( theKey.GetLength() == kKeyLength );

	if (FindElement(theKey, kFalse))
		{
		*cost = ReadCost();
		return kTrue;
		}
	else
		{
		return kFalse;
		}
}

/******************************************************************************
 SetItemCost

	Returns kFalse if the item is not in the list.

 ******************************************************************************/

JBoolean
ItemCost::SetItemCost
	(
	const JString&	theKey,
	const JFloat	cost
	)
{
	assert( !itsLockForIterFlag );
	assert( theKey.GetLength() == kKeyLength );

	if (FindElement(theKey, kFalse))
		{
		WriteCost(cost);
		return kTrue;
		}
	else
		{
		return kFalse;
		}
}

/******************************************************************************
 GetNextItem

 ******************************************************************************/

JBoolean
ItemCost::GetNextItem
	(
	JString*	theKey,
	JFloat*		cost
	)
{
	assert( itsLockForIterFlag );

	if (MoveToNextElement(&itsHashTableIndex))
		{
		*theKey = ReadKey();
		*cost   = ReadCost();
		return kTrue;
		}
	else
		{
		return kFalse;
		}
}

/******************************************************************************
 LockForIteration

	Lock ourselves and prepare to iterate over all the items that have been added.

 ******************************************************************************/

void
ItemCost::LockForIteration()
{
	assert( !itsLockForIterFlag );

	itsLockForIterFlag = kTrue;
	itsHashTableIndex  = 0;
}

/******************************************************************************
 Hash (private)

	Return a hash table index based on the given key.

 ******************************************************************************/

JIndex
ItemCost::Hash
	(
	const JString& theKey
	)
	const
{
	JIndex hashIndex = 0;

	JKLRand r(0);
	for (JIndex i=1; i<=kKeyLength; i++)
		{
		const long seed = theKey.GetCharacter(i);
		r.SetSeed(seed + hashIndex);
		hashIndex = r.UniformULong(1, GetHashTableSize());
		}

	return hashIndex;
}

/******************************************************************************
 CompareFromFile (private)

	Compare the given key with the key at the file read mark.
	Not inline because it is virtual.

 ******************************************************************************/

JBoolean
ItemCost::CompareFromFile
	(
	const JString& theKey
	)
	const
{
	JString fileKey = ReadKey();
	return JConvertToBoolean( theKey == fileKey );
}

/******************************************************************************
 InitializeElement (private)

	Initialize the element at the file write mark.
	Not inline because it is virtual.

 ******************************************************************************/

void
ItemCost::InitializeElement
	(
	const JString& theKey
	)
{
	WriteKey(theKey);
}

/******************************************************************************
 ReadKey (private)

	*** Read mark must already be set correctly.

 ******************************************************************************/

JString
ItemCost::ReadKey()
	const
{
	iostream* theStream = GetFile();
	JCharacter keyData[ kKeyLength+1 ];
	theStream->read(keyData, kKeyLength);
	keyData[ kKeyLength ] = '\0';
	return JString(keyData);
}

/******************************************************************************
 WriteKey (private)

	*** Write mark must already be set correctly.

 ******************************************************************************/

void
ItemCost::WriteKey
	(
	const JString& theKey
	)
{
	iostream* theStream = GetFile();
	theStream->write(theKey, kKeyLength);
}

/******************************************************************************
 ReadCost (private)

	*** Read mark must already be set correctly.

 ******************************************************************************/

JFloat
ItemCost::ReadCost()
	const
{
	JFloat cost;

	iostream* theStream = GetFile();
	theStream->read(static_cast<char*>(&cost), kCostLength);

	// If we read the last element of the file, eof occurs.
	// This needs to be cleared before we can proceed.

	theStream->clear();

	return cost;
}

/******************************************************************************
 WriteCost (private)

	*** Write mark must already be set correctly.

 ******************************************************************************/

void
ItemCost::WriteCost
	(
	const JFloat cost
	)
{
	iostream* theStream = GetFile();
	theStream->write(static_cast<const char*>(&cost), kCostLength);
}
