/******************************************************************************
 MaxX2.cc

	This class implements the Genetic Algorithm for the function -(x-3.4)^2 + 25.

	BASE CLASS = JGeneticString

	Written by John Lindal.

 ******************************************************************************/

#include "MaxX2.h"
#include <jMath.h>
#include <jAssert.h>

// size of each individual -- must be less than # of bits in long

const JSize individualSize    = 16;
const JIndex itsMaxIndividual = JRound(pow(2.0,individualSize)) - 1;

// information about function domain

const JFloat itsMinValue = -1.0;
const JFloat itsMaxValue = 8.0;
const JFloat itsSpan     = itsMaxValue - itsMinValue;

/******************************************************************************
 Constructor

 ******************************************************************************/

MaxX2::MaxX2()
	:
	JGeneticString( individualSize )
{
	itsPopulation = new Population(20);
	assert( itsPopulation != NULL );

	itsOffspring = new Population(20);
	assert( itsOffspring != NULL );
}

/******************************************************************************
 Destructor

 ******************************************************************************/

MaxX2::~MaxX2()
{
	delete itsPopulation;
	delete itsOffspring;
}

/******************************************************************************
 CreateNewPopulation

 ******************************************************************************/

void
MaxX2::CreateNewPopulation
	(
	const JSize populationSize
	)
{
	ClearBestFitness();
	itsBestIndividual = itsMinValue;

	AdjustPopulationSize(itsPopulation, populationSize);
	itsOffspring->RemoveAll();

	// generate random individuals

	for (JIndex i=1; i<=populationSize; i++)
		{
		itsPopulation->SetElement(i, (GetRNG()).UniformLong(0, itsMaxIndividual));
		}
}

/******************************************************************************
 CalcIndividualFitness

 ******************************************************************************/

JFloat
MaxX2::CalcIndividualFitness
	(
	const JIndex index
	)
	const
{
	const long individual = itsPopulation->GetElement(index);
	const JFloat value    = ConvertToFunctionDomain(individual);
	return CalculateFunction(value);
}

/******************************************************************************
 ScaleFitness

 ******************************************************************************/

JBoolean
MaxX2::ScaleFitness
	(
	const JSize			thePopulationSize,
	JPopulationFitness*	thePopulationFitness,
	const JFloat		theMinFitness,
	const JFloat		theAvgFitness,
	const JFloat		theMaxFitness
	)
{
	for (JIndex i=1; i<=thePopulationSize; i++)
		{
		const JFloat scaledFitness = sqrt(thePopulationFitness->GetElement(i));
		thePopulationFitness->SetElement(i, scaledFitness);
		}

	return kTrue;
}

/******************************************************************************
 Mate

 ******************************************************************************/

void
MaxX2::Mate
	(
	const JIndex index1,
	const JIndex index2,
	const JIndex crossoverPosition
	)
{
	const long individual1 = itsPopulation->GetElement(index1);
	const long individual2 = itsPopulation->GetElement(index2);

	long mask = 1;
	for (JIndex i=2; i<=crossoverPosition; i++)
		{
		mask = (mask << 1) + 1;
		}

	const long offspring = (individual1 & mask) + (individual2 & ~mask);

	itsOffspring->AppendElement(offspring);
}

/******************************************************************************
 Mutate

 ******************************************************************************/

void
MaxX2::Mutate
	(
	const JIndex index,
	const JIndex position
	)
{
	long individual = itsOffspring->GetElement(index);

	const long mask = 1 << (position-1);
	individual = individual ^ mask;

	itsOffspring->SetElement(index, individual);
}

/******************************************************************************
 ReplaceOldPopulation

 ******************************************************************************/

void
MaxX2::ReplaceOldPopulation()
{
	*itsPopulation = *itsOffspring;
	itsOffspring->RemoveAll();
}

/******************************************************************************
 SaveBestIndividual

 ******************************************************************************/

void
MaxX2::SaveBestIndividual
	(
	const JIndex bestIndex
	)
{
	const long individual = itsPopulation->GetElement(bestIndex);
	itsBestIndividual     = ConvertToFunctionDomain(individual);
}

/******************************************************************************
 GetBestIndividual

 ******************************************************************************/

JFloat
MaxX2::GetBestIndividual()
	const
{
	return itsBestIndividual;
}

/******************************************************************************
 AdjustPopulationSize (private)

	Adjust the number of individuals in the given population.

 ******************************************************************************/

void
MaxX2::AdjustPopulationSize
	(
	Population*	thePopulation,
	const JSize	populationSize
	)
{
	while (thePopulation->GetElementCount() < populationSize)
		{
		thePopulation->AppendElement(0);
		}
	while (thePopulation->GetElementCount() > populationSize)
		{
		thePopulation->RemoveElement(thePopulation->GetElementCount());
		}
}

/******************************************************************************
 ConvertToFunctionDomain (private)

	Convert the given individual to a value in the function domain.

 ******************************************************************************/

JFloat
MaxX2::ConvertToFunctionDomain
	(
	const long individual
	)
	const
{
	return itsMinValue + itsSpan * ((double) individual) / ((double) itsMaxIndividual);
}

/******************************************************************************
 CalculateFunction (private)

	Calculate the value of the function to maximize.

 ******************************************************************************/

JFloat
MaxX2::CalculateFunction
	(
	const JFloat value
	)
	const
{
	return 25.0 - pow(value-3.4, 2.0);
}
